/*
 * timestamp.c -- lib for get/set interface transmit timestamp.
 */
#include <sys/ioctl.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include <netinet/in.h>
#include <net/if.h>
#include "timestamp.h"

int get_sockfd() {
	int sockfd = -1;
	if ((sockfd = socket(PF_INET, SOCK_DGRAM, 0)) == -1) {
		perror("user: socket creating failed");
		return (-1);
	}
	return sockfd;
}

unsigned long get_tx_timestamp(int fd, char *ifname) {
	struct ifreq ifr;
	struct sockaddr_in *saddr;

	if (fd>=0) {
        strcpy(ifr.ifr_name, ifname);
        ifr.ifr_addr.sa_family = AF_INET;
		/* get tx timestamp */
		if (ioctl(fd, SIOCGIFTTIMESTAMP, &ifr)==0){
			saddr = (struct sockaddr_in *)&ifr.ifr_addr;
			return saddr->sin_addr.s_addr;
		}
	}
	return 0;
}

int set_tx_timestamp(int fd, char *ifname, unsigned long timestamp) {
    struct ifreq ifr;
    struct sockaddr_in *saddr = (struct sockaddr_in *)&ifr.ifr_addr;

	if (fd>=0) {
        strcpy(ifr.ifr_name, ifname);
        ifr.ifr_addr.sa_family = AF_INET;
        saddr->sin_addr.s_addr = timestamp;
		/* set tx timestamp */
		if (ioctl(fd, SIOCSIFTTIMESTAMP, &ifr)==0){
			return 0;
		}
	}
	return -1;
}

