#include "upnpd.h"
#include "tool.h"
#include "gateway.h"
#include "sample_util.h"
#include "dll.h"
#include "action.h"
#include <nvram.h>

/********************************************************
 *		Global Variable Declaration 		*
 ********************************************************/

extern struct GateEnv gate;

DLL start, end;

char *IGDDeviceType[] = {"urn:schemas-upnp-org:device:InternetGatewayDevice:1",
			"urn:schemas-upnp-org:device:WANDevice:1",
			"urn:schemas-upnp-org:device:WANConnectionDevice:1",
			"urn:schemas-upnp-org:device:LANDevice:1"};
char *IGDServiceType[] = {"urn:schemas-microsoft-com:service:OSInfo:1",
                        "urn:schemas-upnp-org:service:WANCommonInterfaceConfig:1",
			"urn:schemas-upnp-org:service:WANIPConnection:1",
			"urn:schemas-upnp-org:service:Layer3Forwarding:1",
			"urn:schemas-upnp-org:service:WANEthernetLinkConfig:1",
			"urn:schemas-upnp-org:service:LANHostConfigManagement:1"};



/* Global arrays for storing <service> OSInfo  variable names, values, and defaults */
char *osi_varname[] = {"OSMajorVersion",
		       "OSMinorVersion",
		       "OSBuildNumber",
		       "OSMachineName"};
char osi_varval[DS_OSI_VARCOUNT][DS_MAX_VAL_LEN];
char *osi_varval_def[] = {"2","2","14","LinuxBase"};

/* arrays for storing WANCommonInterfaceConfig's variable names , values, and defaults */
char *cic_varname[] = {	"WANAccessType",
			"Layer1UpstreamMaxBitRate",
			"Layer1DownstreamMaxBitRate",
			"PhysicalLinkStatus",
			"WANAccessProvider",
			"MaximumActiveConnections",
			"NumberOfActiveConnections",
			"ActiveConnectionDeviceContainer",
			"ActiveConnectionServiceID",
			"TotalBytesSent",
			"TotalBytesReceived",
			"TotalPacketsSent",
			"TotalPacketsReceived",
			"X_PersonalFirewallEnabled",
			"X_Uptime",
			"EnableForInternet"};

char cic_varval[DS_WANCIC_VARCOUNT][DS_MAX_VAL_LEN];
char *cic_varval_def[] = {"Ethernet",  	/* WANAccessType : DSL POTS Cable Ethernet */
			  "10000000",
			  "10000000",
			  "Up",//  "Up",	/* PhysicalLinkStatus : Up Down Initializing Unavailable */
			  "WANAccessProvider",
			  "8",
			  "0",
			  "ActiveConnectionDeviceContainer",
			  "ActiveConnectionServiceID",
			  "100",		/*TotalBytesSent*/
			  "100",          /*TotalBytesReceived*/
			  "100",
			  "100",
			   "1",  // test
			  "0",
			  "1"};


/* arrays for storing WANIPConnction's variable names , values, and defaults */
char *ipconn_varname[] = {"ConnectionType",
			  "PossibleConnectionTypes",
			  "ConnectionStatus",
			  "Uptime",
			  "RSIPAvailable",
			  "NATEnabled",
			  "X_Name",
			  "LastConnectionError",
			  "ExternalIPAddress",
			  "RemoteHost",
			  "ExternalPort",
			  "InternalPort",
			  "PortMappingProtocol",
			  "InternalClient",
			  "PortMappingDescription",
			  "PortMappingEnabled",
			  "PortMappingLeaseDuration",
			  "X_PortMappingIndex",
			  "PortMappingNumberOfEntries"};


char ipconn_varval[DS_WANIPConn_VARCOUNT][DS_MAX_VAL_LEN];
char *ipconn_varval_def[] = {"IP_Routed",
			     "IP_Routed",
			     "Disconnected",   //Connected
			     "0",		/* uptime*/
			     "0",		/* RSIPAvailable */
			     "1", 		/* nat enabled */
			     "Local Area Connection",
			     "ERROR_NONE",
			     "0.0.0.0",
			     "0.0.0.0",
			     "110",
			     "110",
			     "TCP",
			     "0.0.0.0",
			     "POP3",
			     "0",
			     "0",
			     "0",
	//		     "1"};   // modi by john,    at powerup,
	                             //PortMappingNumberOfEntries must be 0
			     "0"};

/* arrays for storing Layer3Forwarding's variable names , values, and defaults */
char *l3f_varname[] = { "DefaultConnectionService"};
#define L3F_DEFAULTSVR_LEN 200
char l3f_varval[DS_L3F_VARCOUNT][L3F_DEFAULTSVR_LEN];
char *l3f_varval_def[] = {"Default"};

/* arrays for storing WANEthernetLinkConfig's variable names , values, and defaults */
char *elc_varname[] = {"EthernetLinkStatus"};
char elc_varval[DS_WANELC_VARCOUNT][DS_MAX_VAL_LEN];
char *elc_varval_def[] = {"Up"};

/* arrays for storing LANHostConfigManagement's variable names , values, and defaults */
char *hcm_varname[] = {"DHCPRelay",
			"DHCPServerConfigurable",
			"DNSServers",
			"DomainName",
			"IPRouters",
			"MaxAddress",
			"MinAddress",
			"ReservedAddress",
			"SubnetMask"};
char hcm_varval[DS_LANHostCM_VARCOUNT][DS_MAX_VAL_LEN];
char *hcm_varval_def[] = {"0",
			  "0",
			  "0.0.0.0",
			  "0",
			  "0",
			  "0.0.0.0",
			  "0.0.0.0",
			  "0.0.0.0",
			  "255.255.255.0"};





/* Global structure for storing the state table for this device */
struct IGDService ds_service_table[6];


/* Mutex for protecting the global state table data
   in a multi-threaded, asynchronous environment.
   All functions should lock this mutex before reading
   or writing the state table data. */
pthread_mutex_t DSDevMutex= PTHREAD_MUTEX_INITIALIZER;




/********************************************************************************
 * IGDDeviceStateTableInit
 *
 * Description:
 *       Initialize the device state table for
 * 	 this IGDDevice, pulling identifier info
 *       from the description Document.  Note that
 *       knowledge of the service description is
 *       assumed.  State table variables and default
 *       values are currently hard coded in this file
 *       rather than being read from service description
 *       documents.
 *
 * Parameters:
 *   DescDocName -- The description document name
 *   DescDocURL -- The description document URL
 *
 ********************************************************************************/
int IGDDeviceStateTableInit(char* DescDocURL, char* DescDocName)
{
    IXML_Document *DescDoc=NULL;
    int ret = UPNP_E_SUCCESS;
    char *servid_osi=NULL, *evnturl_osi=NULL, *ctrlurl_osi=NULL;
    char *servid_cic=NULL, *evnturl_cic=NULL, *ctrlurl_cic=NULL;
    char *servid_ipconn=NULL, *evnturl_ipconn=NULL, *ctrlurl_ipconn=NULL;
    char *servid_l3f=NULL, *evnturl_l3f=NULL, *ctrlurl_l3f=NULL;
    char *servid_elc=NULL, *evnturl_elc=NULL, *ctrlurl_elc=NULL;
    char *servid_hcm=NULL, *evnturl_hcm=NULL, *ctrlurl_hcm=NULL;
    char *udn=NULL;
    char *udn2=NULL;
    char *udn3=NULL;
    char *udn4=NULL;
    char fullurl[256];
    int i;

    sprintf(fullurl,"%s/%s",DescDocURL,DescDocName);
	printf("fullurl %s \n", fullurl);

    if (UpnpDownloadXmlDoc(fullurl, &DescDoc) != UPNP_E_SUCCESS) {
//    if (UpnpDownloadXmlDoc(DescDocURL, &DescDoc) != UPNP_E_SUCCESS) {

	printf("DeviceStateTableInit -- Error Parsing %s\n", DescDocURL);
	ret = UPNP_E_INVALID_DESC;
    }

#if 1
/*
	2005.5.10 john
	OSInfo service lead to igd port mapping setting dialog in networking connection can not open
*/

#if 0
    /* Find the OSInfo Service identifiers */
    if (!SampleUtil_FindAndParseService(DescDoc, DescDocURL, IGDServiceType[DS_SERVICE_OSInfo], &servid_osi, &evnturl_osi, &ctrlurl_osi)) {
	printf("IGDDeviceStateTableInit -- Error: Could not find Service: %s\n", IGDServiceType[DS_SERVICE_OSInfo]);

	if (servid_osi) free(servid_osi);
	if (evnturl_osi) free(evnturl_osi);
	if (ctrlurl_osi) free(ctrlurl_osi);

	return(UPNP_E_INVALID_DESC);
    }
#endif
    /* Find the Common Interface Config Service identifiers */
    if (!SampleUtil_FindAndParseService(DescDoc, DescDocURL, IGDServiceType[DS_SERVICE_WANCommonInterfaceConfig], &servid_cic, &evnturl_cic, &ctrlurl_cic)) {
	printf("IGDDeviceStateTableInit -- Error: Could not find Service: %s\n", IGDServiceType[DS_SERVICE_WANCommonInterfaceConfig]);

	if (servid_cic) free(servid_cic);
	if (evnturl_cic) free(evnturl_cic);
	if (ctrlurl_cic) free(ctrlurl_cic);

	return(UPNP_E_INVALID_DESC);
    }

    /* Find the IP Connection identifiers */
    if (!SampleUtil_FindAndParseService(DescDoc, DescDocURL, IGDServiceType[DS_SERVICE_WANIPConnection], &servid_ipconn, &evnturl_ipconn, &ctrlurl_ipconn)) {
	printf("IGDDeviceStateTableInit -- Error: Could not find Service: %s\n", IGDServiceType[DS_SERVICE_WANIPConnection]);

	if (servid_ipconn) free(servid_ipconn);
	if (evnturl_ipconn) free(evnturl_ipconn);
	if (ctrlurl_ipconn) free(ctrlurl_ipconn);

	return(UPNP_E_INVALID_DESC);
    }


    /* Find the Layer3 Forwarding identifiers */
    if (!SampleUtil_FindAndParseService(DescDoc, DescDocURL, IGDServiceType[DS_SERVICE_Layer3Forwarding], &servid_l3f, &evnturl_l3f, &ctrlurl_l3f)) {
	printf("IGDDeviceStateTableInit -- Error: Could not find Service: %s\n", IGDServiceType[DS_SERVICE_Layer3Forwarding]);

	if (servid_l3f) free(servid_l3f);
	if (evnturl_l3f) free(evnturl_l3f);
	if (ctrlurl_l3f) free(ctrlurl_l3f);

	return(UPNP_E_INVALID_DESC);
    }

    /* Find the WANEthernetLinkConfig identifiers */
    if (!SampleUtil_FindAndParseService(DescDoc, DescDocURL, IGDServiceType[DS_SERVICE_WANEthernetLinkConfig], &servid_elc, &evnturl_elc, &ctrlurl_elc)) {
	printf("IGDDeviceStateTableInit -- Error: Could not find Service: %s\n", IGDServiceType[DS_SERVICE_WANEthernetLinkConfig]);

	if (servid_elc) free(servid_elc);
	if (evnturl_elc) free(evnturl_elc);
	if (ctrlurl_elc) free(ctrlurl_elc);

	return(UPNP_E_INVALID_DESC);
    }


    /* Find the LANHostConfigManagement identifiers */
    if (!SampleUtil_FindAndParseService(DescDoc, DescDocURL, IGDServiceType[DS_SERVICE_LANHostConfigManagement], &servid_hcm, &evnturl_hcm, &ctrlurl_hcm)) {
	printf("IGDDeviceStateTableInit -- Error: Could not find Service: %s\n", IGDServiceType[DS_SERVICE_LANHostConfigManagement]);

	if (servid_hcm) free(servid_hcm);
	if (evnturl_hcm) free(evnturl_hcm);
	if (ctrlurl_hcm) free(ctrlurl_hcm);

	return(UPNP_E_INVALID_DESC);
    }


    udn = SampleUtil_GetDocumentItem(DescDoc, "UDN" ,0);
    udn2 = SampleUtil_GetDocumentItem(DescDoc, "UDN" ,1);
    udn3 = SampleUtil_GetDocumentItem(DescDoc, "UDN" ,2);
    udn4 = SampleUtil_GetDocumentItem(DescDoc, "UDN" ,3);
#if 0
    strcpy(ds_service_table[DS_SERVICE_OSInfo].UDN, udn);
    strcpy(ds_service_table[DS_SERVICE_OSInfo].ServiceId, servid_osi);
    strcpy(ds_service_table[DS_SERVICE_OSInfo].ServiceType, IGDServiceType[DS_SERVICE_OSInfo]);
    ds_service_table[DS_SERVICE_OSInfo].VariableCount=DS_OSI_VARCOUNT;
    for (i=0; i<ds_service_table[DS_SERVICE_OSInfo].VariableCount; i++) {
	ds_service_table[DS_SERVICE_OSInfo].VariableName[i] = osi_varname[i];
	ds_service_table[DS_SERVICE_OSInfo].VariableStrVal[i] = osi_varval[i];
	strcpy(ds_service_table[DS_SERVICE_OSInfo].VariableStrVal[i], osi_varval_def[i]);
    }
#endif
    strcpy(ds_service_table[DS_SERVICE_WANCommonInterfaceConfig].UDN, udn2);
    strcpy(ds_service_table[DS_SERVICE_WANCommonInterfaceConfig].ServiceId, servid_cic);
    strcpy(ds_service_table[DS_SERVICE_WANCommonInterfaceConfig].ServiceType, IGDServiceType[DS_SERVICE_WANCommonInterfaceConfig]);
    ds_service_table[DS_SERVICE_WANCommonInterfaceConfig].VariableCount=DS_WANCIC_VARCOUNT;
    for (i=0; i<ds_service_table[DS_SERVICE_WANCommonInterfaceConfig].VariableCount; i++) {
	ds_service_table[DS_SERVICE_WANCommonInterfaceConfig].VariableName[i] = cic_varname[i];
	ds_service_table[DS_SERVICE_WANCommonInterfaceConfig].VariableStrVal[i] = cic_varval[i];
	strcpy(ds_service_table[DS_SERVICE_WANCommonInterfaceConfig].VariableStrVal[i], cic_varval_def[i]);
   }


    strcpy(ds_service_table[DS_SERVICE_WANIPConnection].UDN, udn3);
    strcpy(ds_service_table[DS_SERVICE_WANIPConnection].ServiceId, servid_ipconn);
    strcpy(ds_service_table[DS_SERVICE_WANIPConnection].ServiceType, IGDServiceType[DS_SERVICE_WANIPConnection]);
    ds_service_table[DS_SERVICE_WANIPConnection].VariableCount=DS_WANIPConn_VARCOUNT;
    for (i=0; i<ds_service_table[DS_SERVICE_WANIPConnection].VariableCount; i++) {
	ds_service_table[DS_SERVICE_WANIPConnection].VariableName[i] = ipconn_varname[i];
	ds_service_table[DS_SERVICE_WANIPConnection].VariableStrVal[i] = ipconn_varval[i];
	strcpy(ds_service_table[DS_SERVICE_WANIPConnection].VariableStrVal[i], ipconn_varval_def[i]);
    }

    strcpy(ds_service_table[DS_SERVICE_Layer3Forwarding].UDN, udn);
    strcpy(ds_service_table[DS_SERVICE_Layer3Forwarding].ServiceId, servid_l3f);
    strcpy(ds_service_table[DS_SERVICE_Layer3Forwarding].ServiceType, IGDServiceType[DS_SERVICE_Layer3Forwarding]);
    ds_service_table[DS_SERVICE_Layer3Forwarding].VariableCount=DS_L3F_VARCOUNT;
    for (i=0; i<ds_service_table[DS_SERVICE_Layer3Forwarding].VariableCount; i++) {
	ds_service_table[DS_SERVICE_Layer3Forwarding].VariableName[i] = l3f_varname[i];
	ds_service_table[DS_SERVICE_Layer3Forwarding].VariableStrVal[i] = l3f_varval[i];
	strcpy(ds_service_table[DS_SERVICE_Layer3Forwarding].VariableStrVal[i], l3f_varval_def[i]);
    }
// layer3forwarding default service must init
//  eg. char l3f_defaultservice[]="uuid:44f5824f-c57d-418c-a131-f22b34e14111:WANConnectionDevice:1,urn:upnp-org:serviceId:WANPPPConn1";
	//warning ! assuming that all string len not large than L3F_DEFAULTSVR_LEN
    char fs[L3F_DEFAULTSVR_LEN]="default";
     if ((strlen(udn3)+strlen(ds_service_table[DS_SERVICE_WANIPConnection].ServiceId)+25)<200) {
     		sprintf(fs,"%s:WANConnectionDevice:1,%s",udn3,ds_service_table[DS_SERVICE_WANIPConnection].ServiceId);
    		strcpy(ds_service_table[DS_SERVICE_Layer3Forwarding].VariableStrVal[DS_L3F_DefaultConnService],fs);
     	}
     //printf("default services=%s\n",ds_service_table[DS_SERVICE_Layer3Forwarding].VariableStrVal[DS_L3F_DefaultConnService]);


    strcpy(ds_service_table[DS_SERVICE_WANEthernetLinkConfig].UDN, udn3);
    strcpy(ds_service_table[DS_SERVICE_WANEthernetLinkConfig].ServiceId, servid_elc);
    strcpy(ds_service_table[DS_SERVICE_WANEthernetLinkConfig].ServiceType, IGDServiceType[DS_SERVICE_WANEthernetLinkConfig]);
    ds_service_table[DS_SERVICE_WANEthernetLinkConfig].VariableCount=DS_WANELC_VARCOUNT;
    for (i=0; i<ds_service_table[DS_SERVICE_WANEthernetLinkConfig].VariableCount; i++) {
	ds_service_table[DS_SERVICE_WANEthernetLinkConfig].VariableName[i] = elc_varname[i];
	ds_service_table[DS_SERVICE_WANEthernetLinkConfig].VariableStrVal[i] = elc_varval[i];
	strcpy(ds_service_table[DS_SERVICE_WANEthernetLinkConfig].VariableStrVal[i], elc_varval_def[i]);
    }

    strcpy(ds_service_table[DS_SERVICE_LANHostConfigManagement].UDN, udn4);
    strcpy(ds_service_table[DS_SERVICE_LANHostConfigManagement].ServiceId, servid_hcm);
    strcpy(ds_service_table[DS_SERVICE_LANHostConfigManagement].ServiceType, IGDServiceType[DS_SERVICE_LANHostConfigManagement]);
    ds_service_table[DS_SERVICE_LANHostConfigManagement].VariableCount=DS_LANHostCM_VARCOUNT;
    for (i=0; i<ds_service_table[DS_SERVICE_LANHostConfigManagement].VariableCount; i++) {
	ds_service_table[DS_SERVICE_LANHostConfigManagement].VariableName[i] = hcm_varname[i];
	ds_service_table[DS_SERVICE_LANHostConfigManagement].VariableStrVal[i] = hcm_varval[i];
	strcpy(ds_service_table[DS_SERVICE_LANHostConfigManagement].VariableStrVal[i], hcm_varval_def[i]);
    }


    if (udn) free(udn);
    if (udn2) free(udn2);
    if (udn3) free(udn3);
    if (udn4) free(udn4);

    if (servid_osi) free(servid_osi);
    if (evnturl_osi) free(evnturl_osi);
    if (ctrlurl_osi) free(ctrlurl_osi);

    if (servid_cic) free(servid_cic);
    if (evnturl_cic) free(evnturl_cic);
    if (ctrlurl_cic) free(ctrlurl_cic);

    if (servid_ipconn) free(servid_ipconn);
    if (evnturl_ipconn) free(evnturl_ipconn);
    if (ctrlurl_ipconn) free(ctrlurl_ipconn);

    if (servid_l3f) free(servid_l3f);
    if (evnturl_l3f) free(evnturl_l3f);
    if (ctrlurl_l3f) free(ctrlurl_l3f);

    if (servid_elc) free(servid_elc);
    if (evnturl_elc) free(evnturl_elc);
    if (ctrlurl_elc) free(ctrlurl_elc);

    if (servid_hcm) free(servid_hcm);
    if (evnturl_hcm) free(evnturl_hcm);
    if (ctrlurl_hcm) free(ctrlurl_hcm);

#endif

    return(ret);
}



/********************************************************************************
 * IGDDeviceHandleSubscriptionRequest
 *
 * Description:
 *       Called during a subscription request callback.  If the
 *       subscription request is for this device and either its
 *       control service or picture service, then accept it.
 *
 * Parameters:
 *   sr_event -- The subscription request event structure
 *
 ********************************************************************************/
int IGDDeviceHandleSubscriptionRequest(struct Upnp_Subscription_Request *sr_event)
{
    int i;
    IXML_Document *PropSet;

    pthread_mutex_lock(&DSDevMutex);

    for (i=0; i<DS_SERVICE_SERVCOUNT; i++) {
	if ((strcmp(sr_event->UDN,ds_service_table[i].UDN) == 0) &&
	    (strcmp(sr_event->ServiceId,ds_service_table[i].ServiceId) == 0)) {
	switch(i){
	case  DS_SERVICE_OSInfo:
            //Another way of doing the same thing.
            PropSet = NULL;
/*            UpnpAddToPropertySet(&PropSet, ds_service_table[DS_SERVICE_OSInfo].VariableName[0],
                    ds_service_table[DS_SERVICE_OSInfo].VariableStrVal[0]);
            UpnpAddToPropertySet(&PropSet, ds_service_table[DS_SERVICE_OSInfo].VariableName[1],
                    ds_service_table[DS_SERVICE_OSInfo].VariableStrVal[1]);
*/
            UpnpAcceptSubscriptionExt(gate.device_handle, sr_event->UDN, sr_event->ServiceId,PropSet,sr_event->Sid);
            ixmlDocument_free(PropSet);
	    break;
	case  DS_SERVICE_WANCommonInterfaceConfig:
            //Another way of doing the same thing.
            PropSet = NULL;
            UpnpAddToPropertySet(&PropSet, ds_service_table[DS_SERVICE_WANCommonInterfaceConfig].VariableName[3],
                    ds_service_table[DS_SERVICE_WANCommonInterfaceConfig].VariableStrVal[3]);
/*            UpnpAddToPropertySet(&PropSet, ds_service_table[DS_SERVICE_WANCommonInterfaceConfig].VariableName[13],
                    ds_service_table[DS_SERVICE_WANCommonInterfaceConfig].VariableStrVal[13]);*/    // x_personalfirewall
            UpnpAcceptSubscriptionExt(gate.device_handle, sr_event->UDN, sr_event->ServiceId,PropSet,sr_event->Sid);
            ixmlDocument_free(PropSet);
	    break;
	case DS_SERVICE_WANIPConnection:
	    //Another way of doing the same thing.
            PropSet = NULL;
            UpnpAddToPropertySet(&PropSet, ds_service_table[DS_SERVICE_WANIPConnection].VariableName[1],
                    ds_service_table[DS_SERVICE_WANIPConnection].VariableStrVal[1]);
            UpnpAddToPropertySet(&PropSet, ds_service_table[DS_SERVICE_WANIPConnection].VariableName[2],
                    ds_service_table[DS_SERVICE_WANIPConnection].VariableStrVal[2]);
		// add by john, support x_name subscribe
            UpnpAddToPropertySet(&PropSet, ds_service_table[DS_SERVICE_WANIPConnection].VariableName[6],
                   ds_service_table[DS_SERVICE_WANIPConnection].VariableStrVal[6]);
            UpnpAddToPropertySet(&PropSet, ds_service_table[DS_SERVICE_WANIPConnection].VariableName[8],
                    ds_service_table[DS_SERVICE_WANIPConnection].VariableStrVal[8]);
            UpnpAddToPropertySet(&PropSet, ds_service_table[DS_SERVICE_WANIPConnection].VariableName[18],
                    ds_service_table[DS_SERVICE_WANIPConnection].VariableStrVal[18]);
            UpnpAcceptSubscriptionExt(gate.device_handle, sr_event->UDN, sr_event->ServiceId,PropSet,sr_event->Sid);
            ixmlDocument_free(PropSet);
	    break;
	case  DS_SERVICE_Layer3Forwarding:
            //Another way of doing the same thing.
            PropSet = NULL;
            UpnpAddToPropertySet(&PropSet, ds_service_table[DS_SERVICE_Layer3Forwarding].VariableName[0],
                    ds_service_table[DS_SERVICE_Layer3Forwarding].VariableStrVal[0]);
            UpnpAcceptSubscriptionExt(gate.device_handle, sr_event->UDN, sr_event->ServiceId,PropSet,sr_event->Sid);
            ixmlDocument_free(PropSet);
	    break;
	case  DS_SERVICE_WANEthernetLinkConfig:
            //Another way of doing the same thing.
            PropSet = NULL;
            UpnpAddToPropertySet(&PropSet, ds_service_table[DS_SERVICE_WANEthernetLinkConfig].VariableName[0],
                    ds_service_table[DS_SERVICE_WANEthernetLinkConfig].VariableStrVal[0]);
            UpnpAcceptSubscriptionExt(gate.device_handle, sr_event->UDN, sr_event->ServiceId,PropSet,sr_event->Sid);
            ixmlDocument_free(PropSet);
	    break;
	case  DS_SERVICE_LANHostConfigManagement:
            //Another way of doing the same thing.
            PropSet = NULL;
/*            UpnpAddToPropertySet(&PropSet, ds_service_table[DS_SERVICE_LANHostConfigManagement].VariableName[0],
                    ds_service_table[DS_SERVICE_LANHostConfigManagement].VariableStrVal[0]);
            UpnpAddToPropertySet(&PropSet, ds_service_table[DS_SERVICE_LANHostConfigManagement].VariableName[1],
                    ds_service_table[DS_SERVICE_LANHostConfigManagement].VariableStrVal[1]);
*/
            UpnpAcceptSubscriptionExt(gate.device_handle, sr_event->UDN, sr_event->ServiceId,PropSet,sr_event->Sid);
            ixmlDocument_free(PropSet);
	    break;
		}
	}
    }

    pthread_mutex_unlock(&DSDevMutex);

    return(1);
}





/********************************************************************************
 * IGDDeviceHandleGetVarRequest
 *
 * Description:
 *       Called during a get variable request callback.
 *	 Check if the request is for this device and its services.
 *	 Check current status and save them to status table
 *	 Responds with the variable value
 *
 * Parameters:
 *   cgv_event -- The control get variable request event structure
 *
 ********************************************************************************/
int IGDDeviceHandleGetVarRequest(struct Upnp_State_Var_Request *cgv_event)
{
    int i, j;
    int gedsar_succeeded = 0;
    cgv_event->CurrentVal = NULL;

    pthread_mutex_lock(&DSDevMutex);

    for (i=0; i<DS_SERVICE_SERVCOUNT; i++) {
	if ((strcmp(cgv_event->DevUDN,ds_service_table[i].UDN)==0) &&
	    (strcmp(cgv_event->ServiceID,ds_service_table[i].ServiceId)==0)) {
	    /* Request for variable in the IGDDevice Service */
	    for (j=0; j< ds_service_table[i].VariableCount; j++) {
		if (strcmp(cgv_event->StateVarName, ds_service_table[i].VariableName[j])==0) {

			//check which StateVarName control point requesting
		    gedsar_succeeded = 1;
		        //find it ,then return success

//		    cgv_event->CurrentVal = (Upnp_DOMString) malloc(sizeof(ds_service_table[i].VariableStrVal[j]));
//		    strcpy(cgv_event->CurrentVal, ds_service_table[i].VariableStrVal[j]);
		    cgv_event->CurrentVal = ixmlCloneDOMString (ds_service_table[i].VariableStrVal[j]);

		        //set value stored in service_table to cgv_event->CurrentVal

		    break;
		}
	    } //end of for loop
	}
    } //end of for loop

    if (gedsar_succeeded) {
	cgv_event->ErrCode = UPNP_E_SUCCESS;
    } else {
	printf("Error in UPNP_CONTROL_GET_VAR_REQUEST callback:\n");
	printf("   Unknown variable name = %s\n",  cgv_event->StateVarName);
	cgv_event->ErrCode = 404;
	strcpy(cgv_event->ErrStr, "Invalid Variable");
    }

    pthread_mutex_unlock(&DSDevMutex);

    return(cgv_event->ErrCode == UPNP_E_SUCCESS);
}


/*******************************************************************************
 * IGDDeviceHandleActionRequest
 *
 * Description:
 *       Called during an action request callback.
 *	 Check which service is requested.
 *	 Perform the action :
 * 			 set/acquire status table, system calls, set/acquire
 * 			 portmapping table
 *	 Respond.
 *
 * Parameters:
 *   ca_event -- The control action request event structure
 *
 ******************************************************************************/



int IGDDeviceHandleActionRequest(struct Upnp_Action_Request *ca_event)
{
    char result_str[500];
    char input_string[DS_MAX_VAL_LEN];
    char old_string[DS_MAX_VAL_LEN];
    char *upnp_enable_conf;
    int user_permission;
    
    upnp_enable_conf = nvram_get("upnp_enable_conf");
    
    /* To handle values in control action request*/
    user_permission=(upnp_enable_conf && *upnp_enable_conf=='1')?1:0; /* 0 <-- not allow ; 1 <-- allow */
    
    if(upnp_enable_conf)
        free(upnp_enable_conf);

    /* Defaults if action not found */
    int action_succeeded = -1;
    int err=401;

    ca_event->ErrCode = 0;
    ca_event->ActionResult = NULL;
    memset(input_string,0,DS_MAX_VAL_LEN);

/********************************
 * WAN Common Interface Config  *
 ********************************/
if ((strcmp(ca_event->DevUDN,ds_service_table[DS_SERVICE_WANCommonInterfaceConfig].UDN)==0) &&
	       (strcmp(ca_event->ServiceID,ds_service_table[DS_SERVICE_WANCommonInterfaceConfig].ServiceId)==0)) {

	/* Request for action in the DsDevice WANCommonInterfaceConfig Service  */


	if (strcmp(ca_event->ActionName, "X_GetICSStatistics") == 0) {

		sprintf(result_str,
			"<u:X_GetICSStatisticsResponse xmlns:u=\"%s\"><TotalBytesSent xmlns:dt=\"urn:schema-microsoft-com:datatypes\" dt:dt=\"ui4\">666</TotalBytesSent><TotalBytesReceived xmlns:dt=\"urn:schema-microsoft-com:datatypes\" dt:dt=\"ui4\">666</TotalBytesReceived><TotalPacketsSent xmlns:dt=\"urn:schema-microsoft-com:datatypes\" dt:dt=\"ui4\">666</TotalPacketsSent><TotalPacketsReceived xmlns:dt=\"urn:schema-microsoft-com:datatypes\" dt:dt=\"ui4\">666</TotalPacketsReceived><Layer1DownstreamMaxBitRate xmlns:dt=\"urn:schema-microsoft-com:datatypes\" dt:dt=\"ui4\">100000000</Layer1DownstreamMaxBitRate><Uptime xmlns:dt=\"urn:schema-microsoft-com:datatypes\" dt:dt=\"ui4\">500</Uptime></u:X_GetICSStatisticsResponse>",
			IGDServiceType[DS_SERVICE_WANCommonInterfaceConfig]);

		ca_event->ActionResult = ixmlParseBuffer(result_str);

		action_succeeded = 1;

	/* GetTotalBytesSent Action */
	}else if (strcmp(ca_event->ActionName, "GetTotalBytesSent") == 0) {

		sprintf(result_str,
			"<u:%sResponse xmlns:u=\"%s\"><NewTotalBytesSent>%lu</NewTotalBytesSent></u:%sResponse>",
			ca_event->ActionName,
			IGDServiceType[DS_SERVICE_WANCommonInterfaceConfig],
			getTotalBytesSent(),
			ca_event->ActionName);

		ca_event->ActionResult = ixmlParseBuffer(result_str);

		action_succeeded = 1;

	/* GetTotalBytesReceived Action */
	}else if (strcmp(ca_event->ActionName, "GetTotalBytesReceived") == 0) {
	sprintf(result_str,
			"<u:%sResponse xmlns:u=\"%s\"><NewTotalBytesReceived>%lu</NewTotalBytesReceived></u:%sResponse>",
			ca_event->ActionName,
			IGDServiceType[DS_SERVICE_WANCommonInterfaceConfig],
 			getTotalBytesReceived(),
			ca_event->ActionName);
		ca_event->ActionResult = ixmlParseBuffer(result_str);
		action_succeeded = 1;

	/* GetTotalPacketsSent Action */
	}else if (strcmp(ca_event->ActionName, "GetTotalPacketsSent") == 0) {

		sprintf(result_str,
			"<u:%sResponse xmlns:u=\"%s\"><NewTotalPacketsSent>%lu</NewTotalPacketsSent></u:%sResponse>",
			ca_event->ActionName,
			IGDServiceType[DS_SERVICE_WANCommonInterfaceConfig],
			getTotalPacketsSent(),
			ca_event->ActionName);
		ca_event->ActionResult = ixmlParseBuffer(result_str);
		action_succeeded = 1;

	/* GetTotalPacketsReceived Action */
	}else if (strcmp(ca_event->ActionName, "GetTotalPacketsReceived") == 0) {

		sprintf(result_str,
			"<u:%sResponse xmlns:u=\"%s\"><NewTotalPacketsReceived>%lu</NewTotalPacketsReceived></u:%sResponse>",
			ca_event->ActionName,
			IGDServiceType[DS_SERVICE_WANCommonInterfaceConfig],
			getTotalPacketsReceived(),
			ca_event->ActionName);
		ca_event->ActionResult = ixmlParseBuffer(result_str);
		action_succeeded = 1;

	/* GetCommonLinkProperties Action */
	}else if (strcmp(ca_event->ActionName, "GetCommonLinkProperties") == 0) {

		sprintf(result_str,
			"<u:%sResponse xmlns:u=\"%s\"><NewWANAccessType>Ethernet</NewWANAccessType><NewLayer1UpstreamMaxBitRate>%lu</NewLayer1UpstreamMaxBitRate><NewLayer1DownstreamMaxBitRate>%lu</NewLayer1DownstreamMaxBitRate><NewPhysicalLinkStatus>Up</NewPhysicalLinkStatus></u:%sResponse>",
			ca_event->ActionName,
			IGDServiceType[DS_SERVICE_WANCommonInterfaceConfig],
			gate.UpstreamSpeed,
			gate.DownstreamSpeed,	
			ca_event->ActionName);

		ca_event->ActionResult = ixmlParseBuffer(result_str);
		action_succeeded = 1;
	}


/********************************
 *	 WAN IP Connection      *
 ********************************/

} else if ((strcmp(ca_event->DevUDN,ds_service_table[DS_SERVICE_WANIPConnection].UDN)==0) &&
	       (strcmp(ca_event->ServiceID,ds_service_table[DS_SERVICE_WANIPConnection].ServiceId)==0)) {

	/* Request for action in the IGDDevice WAN IP Connection Service  */

	if (strcmp(ca_event->ActionName, "SetConnectionType") == 0) {

		sprintf(result_str, "<u:%sResponse xmlns:u=\"%s\"/>",
		ca_event->ActionName,
		IGDServiceType[DS_SERVICE_WANIPConnection]);
  		ca_event->ActionResult = ixmlParseBuffer(result_str);
		action_succeeded = 1;

	/* GetConnectionTypeInfo Action */
	}else
	if (strcmp(ca_event->ActionName, "GetConnectionTypeInfo") == 0) {
		sprintf(result_str,
			"<u:%sResponse xmlns:u=\"%s\">\
			<NewConnectionType>IP_Routed</NewConnectionType>\
			<NewPossibleConnectionTypes>IP_Routed</NewPossibleConnectionTypes>\
			</u:%sResponse>",
		ca_event->ActionName,
		IGDServiceType[DS_SERVICE_WANIPConnection],
		ca_event->ActionName);
	     	ca_event->ActionResult = ixmlParseBuffer(result_str);
        	action_succeeded = 1;
	/* RequestConnection Action */
	}else if (strcmp(ca_event->ActionName, "RequestConnection") == 0) {
		sprintf(result_str, "<u:%sResponse xmlns:u=\"%s\"/>",
			ca_event->ActionName,
			IGDServiceType[DS_SERVICE_WANIPConnection]);
	  	ca_event->ActionResult = ixmlParseBuffer(result_str);
		action_succeeded = IGDDeviceRequestConnection();
	/* ForceTermination */
	}else if (strcmp(ca_event->ActionName, "ForceTermination") == 0) {
		sprintf(result_str, "<u:%sResponse xmlns:u=\"%s\"/>",
			ca_event->ActionName,
			IGDServiceType[DS_SERVICE_WANIPConnection]);
	  	ca_event->ActionResult = ixmlParseBuffer(result_str);
		action_succeeded = IGDDeviceForceTermination();
	/* GetStatusInfo Action */
	}else if (strcmp(ca_event->ActionName, "GetStatusInfo") == 0) {
		// update Gate.connectionStatus
		getConnectionStatus();
		sprintf(result_str,
			"<u:GetStatusInfoResponse xmlns:u=\"%s\">\
			<NewConnectionStatus>%s</NewConnectionStatus>\
			<NewLastConnectionError>ERROR_NONE</NewLastConnectionError>\
			<NewUptime>%d</NewUptime></u:GetStatusInfoResponse>",
			IGDServiceType[DS_SERVICE_WANIPConnection],
			gate.ConnectionStatus,
			getUptime());
		ca_event->ActionResult = ixmlParseBuffer(result_str);
		action_succeeded = 1;
	/* GetNATRSIPStatus Action */
	}else if (strcmp(ca_event->ActionName, "GetNATRSIPStatus") == 0) {
		sprintf(result_str,
			"<u:%sResponse xmlns:u=\"%s\">\
			<NewRSIPAvailable>0</NewRSIPAvailable>\
			<NewNATEnabled>%d</NewNATEnabled>\
			</u:%sResponse>",
			ca_event->ActionName,
			IGDServiceType[DS_SERVICE_WANIPConnection],
			getNATEnabled(),
			ca_event->ActionName);

		ca_event->ActionResult = ixmlParseBuffer(result_str);
	        action_succeeded = 1;


	/* GetGenericPortMappingEntry Action : get PortMapping according to index */
	}else if (strcmp(ca_event->ActionName, "GetGenericPortMappingEntry") == 0) {
		char *my_pt=SampleUtil_GetDocumentItem(ca_event->ActionRequest
				, "NewPortMappingIndex",0);
		int i=0;
		int len=LinkLength();

		if(my_pt){
			i=atoi(my_pt);
			free(my_pt);
		}

/*john debug*/

		//printf("upnp gateway going to get index %d,len=%d,my_pt=%d \n",i,len,my_pt);
// end of john debug

	        if(i+1>len) {
		       action_succeeded = -2;
	        }
		else
			{
			DLL tmp;
		//puts("Searching....in generic\n");
		tmp=Search(i);
		if(tmp==NULL)
			{
			//puts("no such index");
			action_succeeded = -2;
			}

		sprintf(result_str,
			"<u:%sResponse xmlns:u=\"%s\"><NewRemoteHost></NewRemoteHost><NewExternalPort>%d</NewExternalPort><NewProtocol>%s</NewProtocol><NewInternalPort>%d</NewInternalPort><NewInternalClient>%s</NewInternalClient><NewEnabled>%d</NewEnabled><NewPortMappingDescription>%s</NewPortMappingDescription><NewLeaseDuration>0</NewLeaseDuration></u:%sResponse>",
			ca_event->ActionName,
			IGDServiceType[DS_SERVICE_WANIPConnection],
			tmp->data.ExternalPort,
			tmp->data.Protocol,
			tmp->data.InternalPort,
			tmp->data.InternalClient,
			tmp->data.PortMappingEnabled,
			tmp->data.PortMappingDescription,
			ca_event->ActionName);

		ca_event->ActionResult = ixmlParseBuffer(result_str);

	        action_succeeded = 1;
		}

	} else if (strcmp(ca_event->ActionName, "GetSpecificPortMappingEntry") == 0) {
		char *my_pt=SampleUtil_GetDocumentItem(ca_event->ActionRequest
				, "NewExternalPort",0);
		int ext_port=0;
		char *protocol=SampleUtil_GetDocumentItem(ca_event->ActionRequest, "NewProtocol",0);
		int i=0,len=LinkLength();
		int flag=0;
		DLL tmp;

		if(my_pt){
			ext_port=atoi(my_pt);
			free(my_pt);
		}

		for(i=0;i<len;i++)
		{

			//puts("Searching....in specific\n");
			tmp=Search(i);

			if((tmp->data.ExternalPort==ext_port)
					&&(strcmp(tmp->data.Protocol, protocol)==0)){
			if (tmp->data.RemoteHost == NULL)
				sprintf(result_str,
/*     2005.5.11  john
	  NewRemoteHost  should not exist in response
				"<u:%sResponse xmlns:u=\"%s\"><NewRemoteHost></NewRemoteHost><NewInternalPort>%d</NewInternalPort><NewInternalClient>%s</NewInternalClient><NewEnabled>%d</NewEnabled>	<NewPortMappingDescription>%s</NewPortMappingDescription><NewLeaseDuration>%ld</NewLeaseDuration></u:%sResponse>",
*/
				"<u:%sResponse xmlns:u=\"%s\"><NewInternalPort>%d</NewInternalPort><NewInternalClient>%s</NewInternalClient><NewEnabled>%d</NewEnabled>	<NewPortMappingDescription>%s</NewPortMappingDescription><NewLeaseDuration>%ld</NewLeaseDuration></u:%sResponse>",

				ca_event->ActionName,
				IGDServiceType[DS_SERVICE_WANIPConnection],
				tmp->data.InternalPort,
				tmp->data.InternalClient,
				tmp->data.PortMappingEnabled,
				tmp->data.PortMappingDescription,
				tmp->data.PortMappingLeaseDuration,
				ca_event->ActionName);
			else
				sprintf(result_str,
/*     2005.5.11  john
	  NewRemoteHost  should not exist in response
				"<u:%sResponse xmlns:u=\"%s\"><NewRemoteHost>%s</NewRemoteHost><NewInternalPort>%d</NewInternalPort>	<NewInternalClient>%s</NewInternalClient><NewEnabled>%d</NewEnabled>	<NewPortMappingDescription>%s</NewPortMappingDescription><NewLeaseDuration>%ld</NewLeaseDuration></u:%sResponse>",
*/
				"<u:%sResponse xmlns:u=\"%s\"><NewInternalPort>%d</NewInternalPort><NewInternalClient>%s</NewInternalClient><NewEnabled>%d</NewEnabled>	<NewPortMappingDescription>%s</NewPortMappingDescription><NewLeaseDuration>%ld</NewLeaseDuration></u:%sResponse>",
				ca_event->ActionName,
				IGDServiceType[DS_SERVICE_WANIPConnection],
//				tmp->data.RemoteHost,
				tmp->data.InternalPort,
				tmp->data.InternalClient,
				tmp->data.PortMappingEnabled,
				tmp->data.PortMappingDescription,
				tmp->data.PortMappingLeaseDuration,
				ca_event->ActionName);

				ca_event->ActionResult = ixmlParseBuffer(result_str);
				flag=1;
			} //end of if
		} //end of for

		if(protocol)
			free(protocol);

	if(flag==1)
		action_succeeded = 1;
	else
		action_succeeded = -3;

	} else if (strcmp(ca_event->ActionName, "AddPortMapping") == 0) {
		if(user_permission==0) action_succeeded = -1;
		else
		{
		DLL tmp,node;
		//int i,len=LinkLength();
		int i;
		time_t t;

		tmp=AllocNode();
		tmp->data.RemoteHost=SampleUtil_GetDocumentItem(ca_event->ActionRequest, "NewRemoteHost",0);
		tmp->data.PortMappingLeaseDuration=atol(SampleUtil_GetDocumentItem(ca_event->ActionRequest, "NewLeaseDuration",0));
		tmp->data.ExternalPort=atoi(SampleUtil_GetDocumentItem(ca_event->ActionRequest, "NewExternalPort",0));
		tmp->data.Protocol=SampleUtil_GetDocumentItem(ca_event->ActionRequest, "NewProtocol",0);
		tmp->data.InternalPort=atoi(SampleUtil_GetDocumentItem(ca_event->ActionRequest, "NewInternalPort",0));
		tmp->data.InternalClient=SampleUtil_GetDocumentItem(ca_event->ActionRequest, "NewInternalClient",0);
		tmp->data.PortMappingEnabled=atoi(SampleUtil_GetDocumentItem(ca_event->ActionRequest, "NewEnabled",0));
		tmp->data.PortMappingDescription=SampleUtil_GetDocumentItem(ca_event->ActionRequest, "NewPortMappingDescription",0);
		tmp->data.lease_t=0;


		// Search if the To-Be-Add Portmapping already exist, if do, remove it first
		for(i=0;i<LinkLength();i++)
		{
			node=Search(i);
			if((node->data.ExternalPort==tmp->data.ExternalPort)&&(strcmp(node->data.Protocol, tmp->data.Protocol)==0))
			{
				if (strcmp(node->data.InternalClient,tmp->data.InternalClient)!=0) {
					action_succeeded = -5;
					break;
				}
				else {   //according protocol,  if internalClient equal, must overwrite it
					//cd router require reject the request for add existing portmapping,Carl
					//puts("Found an exist port mapping");
		       		if(node->data.PortMappingEnabled!=0)
						delPortForward(node->data.Protocol,
       						//gate.ExternalIPAddress,
       						getIPAddress(gate.ext_ifname)?:"0.0.0.0",
       						node->data.ExternalPort,
       						node->data.InternalClient,
       						node->data.InternalPort,
       						node->data.RemoteHost);
					Delete(node);
					//				print_node(tmp);
					print_link();
					break;
				}
			}
		}

       	if (action_succeeded != -5)
       	{
	       			// do really work
	       		if(tmp->data.PortMappingEnabled==1)
	       		addPortForward(tmp->data.Protocol,
	       				getIPAddress(gate.ext_ifname)?:"0.0.0.0",
	       				//gate.ExternalIPAddress,
	       				tmp->data.ExternalPort,
	       				tmp->data.InternalClient,
	       				tmp->data.InternalPort,
	       				tmp->data.RemoteHost);
	//		print_node(tmp);
				// Append data to Linked List
				    if(tmp->data.PortMappingLeaseDuration!=0)
				    {
                        time(&t);
                        tmp->data.lease_t=t+tmp->data.PortMappingLeaseDuration;
                    }
		       		Append(tmp);

				print_link();

			/* modify according to package we get from certification record */
			sprintf(result_str, "<u:%sResponse xmlns:u=\"%s\"/>",
				ca_event->ActionName,
				IGDServiceType[DS_SERVICE_WANIPConnection]);
	  		ca_event->ActionResult = ixmlParseBuffer(result_str);
			action_succeeded = 1;
		}
		}
	}
	else if (strcmp(ca_event->ActionName, "DeletePortMapping") == 0) {
	if(user_permission==0)
		action_succeeded = -1;
	else
	{
		//int i, len=LinkLength();
		int i;
		DLL tmp;
		char *my_pt=SampleUtil_GetDocumentItem(ca_event->ActionRequest
				, "NewExternalPort",0);
		int ext_port=atoi(my_pt);
		char *protocol=SampleUtil_GetDocumentItem(ca_event->ActionRequest, "NewProtocol",0);

		if(my_pt)
			free(my_pt);

		// Search if the To-Be-Delete Portmapping already exist,
		for(i=0;i<LinkLength();i++)
		{

			tmp=Search(i);
			if((ext_port==tmp->data.ExternalPort)&&(strcmp(protocol, tmp->data.Protocol)==0))
			{

				//puts("Found an exist port mapping");

				/* Delete Port Mapping before free this node */
		       		//if(tmp->data.PortMappingEnabled!=0)
		       		delPortForward(tmp->data.Protocol,
       				getIPAddress(gate.ext_ifname)?:"0.0.0.0",
       				//gate.ExternalIPAddress,
       				tmp->data.ExternalPort,
       				tmp->data.InternalClient,
       				tmp->data.InternalPort,
       				tmp->data.RemoteHost);

				/* update Linked List */
				Delete(tmp);
				/* update upnptab */
				print_link();
				break;
			}

		  }
		if(protocol)
			free(protocol);

		sprintf(result_str, "<u:%sResponse xmlns:u=\"%s\"/>",
				ca_event->ActionName,
				IGDServiceType[DS_SERVICE_WANIPConnection]);
		ca_event->ActionResult = ixmlParseBuffer(result_str);
		action_succeeded = 1;
		}//end of permission granted
	} else if (strcmp(ca_event->ActionName, "GetExternalIPAddress") == 0) {

		sprintf(result_str,
		"<u:%sResponse xmlns:u=\"%s\">\
		 <NewExternalIPAddress>%s</NewExternalIPAddress>\
	 	 </u:%sResponse>",
		ca_event->ActionName,
		IGDServiceType[DS_SERVICE_WANIPConnection],
		//gate.ExternalIPAddress,
       		getIPAddress(gate.ext_ifname)?:"0.0.0.0",
		ca_event->ActionName);

		ca_event->ActionResult = ixmlParseBuffer(result_str);

	        action_succeeded = 1;
	}
    }

    /* Request for action in the IGDDevice OSInfo Service  */
    else if((strcmp(ca_event->DevUDN,ds_service_table[DS_SERVICE_OSInfo].UDN)==0) &&
            (strcmp(ca_event->ServiceID,ds_service_table[DS_SERVICE_OSInfo].ServiceId)==0))
    {
        /* MagicOn Action */
        if (strcmp(ca_event->ActionName, "MagicOn") == 0)
        {
            sprintf(result_str, "<u:%sResponse xmlns:u=\"%s\"/>",
                ca_event->ActionName,
                IGDServiceType[DS_SERVICE_OSInfo]);
            ca_event->ActionResult = ixmlParseBuffer(result_str);
            action_succeeded = 1;
        }
    }

    /* Request for action in the IGDDevice Layer3Forwarding Service  */
    else if((strcmp(ca_event->DevUDN,ds_service_table[DS_SERVICE_Layer3Forwarding].UDN)==0) &&
               (strcmp(ca_event->ServiceID,ds_service_table[DS_SERVICE_Layer3Forwarding].ServiceId)==0))
    {
        /* GetDefaultConnectionService Action */
        if (strcmp(ca_event->ActionName, "GetDefaultConnectionService") == 0)
        {
            sprintf(result_str, "<u:%sResponse xmlns:u=\"%s\">\
                <NewDefaultConnectionService>%s</NewDefaultConnectionService>\
                </u:%sResponse>",
                ca_event->ActionName,
                IGDServiceType[DS_SERVICE_Layer3Forwarding],
                ds_service_table[DS_SERVICE_Layer3Forwarding].VariableStrVal[DS_L3F_DefaultConnService],
                ca_event->ActionName);
            ca_event->ActionResult = ixmlParseBuffer(result_str);
            action_succeeded = 1;
        }
        /* SetDefaultConnectionService Action */
        else if (strcmp(ca_event->ActionName, "SetDefaultConnectionService") == 0)
        {
            sprintf(result_str, "<u:%sResponse xmlns:u=\"%s\"/>",
                ca_event->ActionName,
                IGDServiceType[DS_SERVICE_Layer3Forwarding]);
            strcpy(ds_service_table[DS_SERVICE_Layer3Forwarding].VariableStrVal[DS_L3F_DefaultConnService],
                SampleUtil_GetDocumentItem(ca_event->ActionRequest, "NewDefaultConnectionService",0));
            ca_event->ActionResult = ixmlParseBuffer(result_str);
            action_succeeded = 1;
        }
    }

    /* Request for action in the IGDDevice WANEthernetLinkConfig Service  */
    else if((strcmp(ca_event->DevUDN,ds_service_table[DS_SERVICE_WANEthernetLinkConfig].UDN)==0) &&
            (strcmp(ca_event->ServiceID,ds_service_table[DS_SERVICE_WANEthernetLinkConfig].ServiceId)==0))
    {
        /* GetEthernetLinkStatus Action */
        if (strcmp(ca_event->ActionName, "GetEthernetLinkStatus") == 0)
        {
            sprintf(result_str, "<u:%sResponse xmlns:u=\"%s\">\
                <NewEthernetLinkStatus>%s</NewEthernetLinkStatus>\
                </u:%sResponse>",
                ca_event->ActionName,
                IGDServiceType[DS_SERVICE_WANEthernetLinkConfig],
                ds_service_table[DS_SERVICE_WANEthernetLinkConfig].VariableStrVal[DS_WANELC_EthernetLinkStatus],
                ca_event->ActionName);
            ca_event->ActionResult = ixmlParseBuffer(result_str);
            action_succeeded = 1;
        }
    }

    /* Request for action in the IGDDevice LANHostConfigManagement Service  */
    else if((strcmp(ca_event->DevUDN,ds_service_table[DS_SERVICE_LANHostConfigManagement].UDN)==0) &&
            (strcmp(ca_event->ServiceID,ds_service_table[DS_SERVICE_LANHostConfigManagement].ServiceId)==0))
    {
        /* GetAddressRange Action */
        if (strcmp(ca_event->ActionName, "GetAddressRange") == 0)
        {
            sprintf(result_str, "<u:%sResponse xmlns:u=\"%s\">\
                <NewMinAddress>%s</NewMinAddress>\
                <NewMaxAddress>%s</NewMaxAddress>\
                </u:%sResponse>",
                ca_event->ActionName,
                IGDServiceType[DS_SERVICE_LANHostConfigManagement],
                ds_service_table[DS_SERVICE_LANHostConfigManagement].VariableStrVal[DS_LANHostCM_MinAddress],
                ds_service_table[DS_SERVICE_LANHostConfigManagement].VariableStrVal[DS_LANHostCM_MinAddress],
                ca_event->ActionName);
            ca_event->ActionResult = ixmlParseBuffer(result_str);
            action_succeeded = 1;
        }
        /* GetDHCPRelay Action */
        else if (strcmp(ca_event->ActionName, "GetDHCPRelay") == 0)
        {
            sprintf(result_str, "<u:%sResponse xmlns:u=\"%s\">\
                <NewDHCPRelay>%s</NewDHCPRelay>\
                </u:%sResponse>",
                ca_event->ActionName,
                IGDServiceType[DS_SERVICE_LANHostConfigManagement],
                ds_service_table[DS_SERVICE_LANHostConfigManagement].VariableStrVal[DS_LANHostCM_DHCPRelay],
                ca_event->ActionName);
            ca_event->ActionResult = ixmlParseBuffer(result_str);
            action_succeeded = 1;
        }
        /* GetDHCPServerConfigurable Action */
        else if (strcmp(ca_event->ActionName, "GetDHCPServerConfigurable") == 0)
        {
            sprintf(result_str, "<u:%sResponse xmlns:u=\"%s\">\
                <NewDHCPServerConfigurable>%s</NewDHCPServerConfigurable>\
                </u:%sResponse>",
                ca_event->ActionName,
                IGDServiceType[DS_SERVICE_LANHostConfigManagement],
                ds_service_table[DS_SERVICE_LANHostConfigManagement].VariableStrVal[DS_LANHostCM_DHCPSrvConfig],
                ca_event->ActionName);
            ca_event->ActionResult = ixmlParseBuffer(result_str);
            action_succeeded = 1;
        }
        /* GetDNSServers Action */
        else if (strcmp(ca_event->ActionName, "GetDNSServers") == 0)
        {
            sprintf(result_str, "<u:%sResponse xmlns:u=\"%s\">\
                <NewDNSServers>%s</NewDNSServers>\
                </u:%sResponse>",
                ca_event->ActionName,
                IGDServiceType[DS_SERVICE_LANHostConfigManagement],
                ds_service_table[DS_SERVICE_LANHostConfigManagement].VariableStrVal[DS_LANHostCM_DNSServers],
                ca_event->ActionName);
            ca_event->ActionResult = ixmlParseBuffer(result_str);
            action_succeeded = 1;
        }
        /* GetDomainName Action */
        else if (strcmp(ca_event->ActionName, "GetDomainName") == 0)
        {
            sprintf(result_str, "<u:%sResponse xmlns:u=\"%s\">\
                <NewDomainName>%s</NewDomainName>\
                </u:%sResponse>",
                ca_event->ActionName,
                IGDServiceType[DS_SERVICE_LANHostConfigManagement],
                ds_service_table[DS_SERVICE_LANHostConfigManagement].VariableStrVal[DS_LANHostCM_DomainName],
                ca_event->ActionName);
            ca_event->ActionResult = ixmlParseBuffer(result_str);
            action_succeeded = 1;
        }
        /* GetIPRoutersList Action */
        else if (strcmp(ca_event->ActionName, "GetIPRoutersList") == 0)
        {
            sprintf(result_str, "<u:%sResponse xmlns:u=\"%s\">\
                <NewIPRouters>%s</NewIPRouters>\
                </u:%sResponse>",
                ca_event->ActionName,
                IGDServiceType[DS_SERVICE_LANHostConfigManagement],
                ds_service_table[DS_SERVICE_LANHostConfigManagement].VariableStrVal[DS_LANHostCM_IPRouters],
                ca_event->ActionName);
            ca_event->ActionResult = ixmlParseBuffer(result_str);
            action_succeeded = 1;
        }
        /* GetReservedAddress Action */
        else if (strcmp(ca_event->ActionName, "GetReservedAddresses") == 0)
        {
            sprintf(result_str, "<u:%sResponse xmlns:u=\"%s\">\
                <NewReservedAddresses>%s</NewReservedAddresses>\
                </u:%sResponse>",
                ca_event->ActionName,
                IGDServiceType[DS_SERVICE_LANHostConfigManagement],
                ds_service_table[DS_SERVICE_LANHostConfigManagement].VariableStrVal[DS_LANHostCM_ReservedAddr],
                ca_event->ActionName);
            ca_event->ActionResult = ixmlParseBuffer(result_str);
            action_succeeded = 1;
        }
        /* GetSubnetMask Action */
        else if (strcmp(ca_event->ActionName, "GetSubnetMask") == 0)
        {
            sprintf(result_str, "<u:%sResponse xmlns:u=\"%s\">\
                <NewSubnetMask>%s</NewSubnetMask>\
                </u:%sResponse>",
                ca_event->ActionName,
                IGDServiceType[DS_SERVICE_LANHostConfigManagement],
                ds_service_table[DS_SERVICE_LANHostConfigManagement].VariableStrVal[DS_LANHostCM_SubnetMask],
                ca_event->ActionName);
            ca_event->ActionResult = ixmlParseBuffer(result_str);
            action_succeeded = 1;
        }
        /* SetAddressRange Action */
        else if (strcmp(ca_event->ActionName, "SetAddressRange") == 0)
        {
            sprintf(result_str, "<u:%sResponse xmlns:u=\"%s\"/>",
                ca_event->ActionName,
                IGDServiceType[DS_SERVICE_LANHostConfigManagement]);
            strcpy(ds_service_table[DS_SERVICE_LANHostConfigManagement].VariableStrVal[DS_LANHostCM_MinAddress],
                SampleUtil_GetDocumentItem(ca_event->ActionRequest, "NewMinAddress",0));
            strcpy(ds_service_table[DS_SERVICE_LANHostConfigManagement].VariableStrVal[DS_LANHostCM_MaxAddress],
                SampleUtil_GetDocumentItem(ca_event->ActionRequest, "NewMaxAddress",0));
            ca_event->ActionResult = ixmlParseBuffer(result_str);
            action_succeeded = 1;
        }
        /* SetDHCPRelay Action */
        else if (strcmp(ca_event->ActionName, "SetDHCPRelay") == 0)
        {
            sprintf(result_str, "<u:%sResponse xmlns:u=\"%s\"/>",
                ca_event->ActionName,
                IGDServiceType[DS_SERVICE_LANHostConfigManagement]);
            strcpy(ds_service_table[DS_SERVICE_LANHostConfigManagement].VariableStrVal[DS_LANHostCM_DHCPRelay],
                SampleUtil_GetDocumentItem(ca_event->ActionRequest, "NewDHCPRelay",0));
            ca_event->ActionResult = ixmlParseBuffer(result_str);
            action_succeeded = 1;
        }
        /* SetDHCPServerConfigurable Action */
        else if (strcmp(ca_event->ActionName, "SetDHCPServerConfigurable") == 0)
        {
            sprintf(result_str, "<u:%sResponse xmlns:u=\"%s\"/>",
                ca_event->ActionName,
                IGDServiceType[DS_SERVICE_LANHostConfigManagement]);
            strcpy(ds_service_table[DS_SERVICE_LANHostConfigManagement].VariableStrVal[DS_LANHostCM_DHCPSrvConfig],
                SampleUtil_GetDocumentItem(ca_event->ActionRequest, "NewDHCPServerConfigurable",0));
            ca_event->ActionResult = ixmlParseBuffer(result_str);
            action_succeeded = 1;
        }
        /* SetDNSServer Action */
        else if (strcmp(ca_event->ActionName, "SetDNSServer") == 0)
        {
            sprintf(result_str, "<u:%sResponse xmlns:u=\"%s\"/>",
                ca_event->ActionName,
                IGDServiceType[DS_SERVICE_LANHostConfigManagement]);
            strcpy(input_string,SampleUtil_GetDocumentItem(ca_event->ActionRequest, "NewDNSServers",0));
            strcpy(old_string,ds_service_table[DS_SERVICE_LANHostConfigManagement].VariableStrVal[DS_LANHostCM_DNSServers]);
            if(0 < strlen(old_string))
                strcat(old_string,",");
            strcat(old_string,input_string);
            strcpy(ds_service_table[DS_SERVICE_LANHostConfigManagement].VariableStrVal[DS_LANHostCM_DNSServers],old_string);
            ca_event->ActionResult = ixmlParseBuffer(result_str);
            action_succeeded = 1;
        }
        /* SetDomainName Action */
        else if (strcmp(ca_event->ActionName, "SetDomainName") == 0)
        {
            sprintf(result_str, "<u:%sResponse xmlns:u=\"%s\"/>",
                ca_event->ActionName,
                IGDServiceType[DS_SERVICE_LANHostConfigManagement]);
            strcpy(ds_service_table[DS_SERVICE_LANHostConfigManagement].VariableStrVal[DS_LANHostCM_DomainName],
                SampleUtil_GetDocumentItem(ca_event->ActionRequest, "NewDomainName",0));
            ca_event->ActionResult = ixmlParseBuffer(result_str);
            action_succeeded = 1;
        }
        /* SetIPRouter Action */
        else if (strcmp(ca_event->ActionName, "SetIPRouter") == 0)
        {
            sprintf(result_str, "<u:%sResponse xmlns:u=\"%s\"/>",
                ca_event->ActionName,
                IGDServiceType[DS_SERVICE_LANHostConfigManagement]);
            strcpy(input_string,SampleUtil_GetDocumentItem(ca_event->ActionRequest, "NewIPRouters",0));
            strcpy(old_string,ds_service_table[DS_SERVICE_LANHostConfigManagement].VariableStrVal[DS_LANHostCM_IPRouters]);
            if(0 < strlen(old_string))
                strcat(old_string,",");
            strcat(old_string,input_string);
            strcpy(ds_service_table[DS_SERVICE_LANHostConfigManagement].VariableStrVal[DS_LANHostCM_IPRouters],old_string);
            ca_event->ActionResult = ixmlParseBuffer(result_str);
            action_succeeded = 1;
        }
        /* SetReservedAddress Action */
        else if (strcmp(ca_event->ActionName, "SetReservedAddress") == 0)
        {
            sprintf(result_str, "<u:%sResponse xmlns:u=\"%s\"/>",
                ca_event->ActionName,
                IGDServiceType[DS_SERVICE_LANHostConfigManagement]);
            strcpy(input_string,SampleUtil_GetDocumentItem(ca_event->ActionRequest, "NewReservedAddresses",0));
            strcpy(old_string,ds_service_table[DS_SERVICE_LANHostConfigManagement].VariableStrVal[DS_LANHostCM_ReservedAddr]);
            if(0 < strlen(old_string))
                strcat(old_string,",");
            strcat(old_string,input_string);
            strcpy(ds_service_table[DS_SERVICE_LANHostConfigManagement].VariableStrVal[DS_LANHostCM_ReservedAddr],old_string);
            ca_event->ActionResult = ixmlParseBuffer(result_str);
            action_succeeded = 1;
        }
        /* SetSubnetMask Action */
        else if (strcmp(ca_event->ActionName, "SetSubnetMask") == 0)
        {
            sprintf(result_str, "<u:%sResponse xmlns:u=\"%s\"/>",
                ca_event->ActionName,
                IGDServiceType[DS_SERVICE_LANHostConfigManagement]);
            strcpy(ds_service_table[DS_SERVICE_LANHostConfigManagement].VariableStrVal[DS_LANHostCM_SubnetMask],
                SampleUtil_GetDocumentItem(ca_event->ActionRequest, "NewSubnetMask",0));
            ca_event->ActionResult = ixmlParseBuffer(result_str);
            action_succeeded = 1;
        }
        /* DeleteDNSServer Action */
        else if (strcmp(ca_event->ActionName, "DeleteDNSServer") == 0)
        {
            sprintf(result_str, "<u:%sResponse xmlns:u=\"%s\"/>",
                ca_event->ActionName,
                IGDServiceType[DS_SERVICE_LANHostConfigManagement]);
            strcpy(input_string,SampleUtil_GetDocumentItem(ca_event->ActionRequest, "NewDNSServers",0));
            if(0 < DeleteDNSServer(input_string))
            {
                ca_event->ActionResult = ixmlParseBuffer(result_str);
                action_succeeded = 1;
            }
            else
            {
                action_succeeded = -4;
            }
        }
        /* DeleteIPRouter Action */
        else if (strcmp(ca_event->ActionName, "DeleteIPRouter") == 0)
        {
            sprintf(result_str, "<u:%sResponse xmlns:u=\"%s\"/>",
                ca_event->ActionName,
                IGDServiceType[DS_SERVICE_LANHostConfigManagement]);
            ds_service_table[DS_SERVICE_LANHostConfigManagement].VariableStrVal[DS_LANHostCM_IPRouters][0]=0;
            ca_event->ActionResult = ixmlParseBuffer(result_str);
            action_succeeded = 1;
        }
        /* DeleteReservedAddress Action */
        else if (strcmp(ca_event->ActionName, "DeleteReservedAddress") == 0)
        {
            sprintf(result_str, "<u:%sResponse xmlns:u=\"%s\"/>",
                ca_event->ActionName,
                IGDServiceType[DS_SERVICE_LANHostConfigManagement]);
            ds_service_table[DS_SERVICE_LANHostConfigManagement].VariableStrVal[DS_LANHostCM_ReservedAddr][0]=0;
            ca_event->ActionResult = ixmlParseBuffer(result_str);
            action_succeeded = 1;
        }
    }
    else{
//       printf("Error in UPNP_CONTROL_ACTION_REQUEST callback:\n");
//	printf("   Unknown UDN = %s or ServiceId = %s\n",  ca_event->DevUDN, ca_event->ServiceID );
    }
    if (action_succeeded > 0) {
	ca_event->ErrCode = UPNP_E_SUCCESS;
    } else if (action_succeeded == -1) {
//	printf("Error in UPNP_CONTROL_ACTION_REQUEST callback:\n");
//	printf("   Unknown ActionName = %s\n",  ca_event->ActionName);
	ca_event->ErrCode = err;
	strcpy(ca_event->ErrStr, "Invalid Action");
    	ca_event->ActionResult = NULL;

   } else if (action_succeeded == -2) {
	ca_event->ErrCode = 713;
	strcpy(ca_event->ErrStr, "SpecifiedArrayIndexInvalid");
    	ca_event->ActionResult = NULL;
   } else if (action_succeeded == -3) {

	ca_event->ErrCode = 714;
	strcpy(ca_event->ErrStr, "NoSuchEntryInArray");
    	ca_event->ActionResult = NULL;

    }
	else if (action_succeeded == -5) {

	ca_event->ErrCode = 718;
	strcpy(ca_event->ErrStr, "ConflictInMappingEntry");
    	ca_event->ActionResult = NULL;

    }
   else {
	printf("Error in UPNP_CONTROL_ACTION_REQUEST callback:\n");
	printf("   Failure while running %s\n",  ca_event->ActionName);

	ca_event->ErrCode = 402;
	strcpy(ca_event->ErrStr, "Invalid Args");
    	ca_event->ActionResult = NULL;
    }

    return(ca_event->ErrCode);
}




/********************************************************************************
 * IGDDeviceSetServiceTableVar
 *
 * Description:
 *       Update the IGDDevice service state table, and notify all subscribed
 *       control points of the updated state.  Note that since this function
 *       blocks on the mutex DSDevMutex, to avoid a hang this function should
 *       not be called within any other function that currently has this mutex
 *       locked.
 *
 * Parameters:
 *   service -- The service number (DS_SERVICE_Layer3Forwarding or DS_SERVICE_PICTURE)
 *   variable -- The variable number (DS_CONTROL_POWER, DS_CONTROL_CHANNEL,
 *                   DS_CONTROL_VOLUME, DS_PICTURE_COLOR, DS_PICTURE_TINT,
 *                   DS_PICTURE_CONTRAST, or DS_PICTURE_BRIGHTNESS)
 *   value -- The string representation of the new value
 *
 ********************************************************************************/
int IGDDeviceSetServiceTableVar(unsigned int service, unsigned int variable, char *value) {
 IXML_Document  *PropSet= NULL;
    if ((service >= DS_SERVICE_SERVCOUNT) || (variable >= ds_service_table[service].VariableCount) || (strlen(value) >= DS_MAX_VAL_LEN)) {
	return(0);
    }

    pthread_mutex_lock(&DSDevMutex);

    strcpy(ds_service_table[service].VariableStrVal[variable], value);

    //Using second utility API
    PropSet= UpnpCreatePropertySet(1,ds_service_table[service].VariableName[variable], ds_service_table[service].VariableStrVal[variable]);
    UpnpNotifyExt(gate.device_handle, ds_service_table[service].UDN, ds_service_table[service].ServiceId,PropSet);
    ixmlDocument_free(PropSet);

    pthread_mutex_unlock(&DSDevMutex);

    return(1);

}


/********************************************************************************
 * IGDDeviceSetServiceTableVars
 *
 * Description:
 *       Update the IGDDevice service state table, and notify all subscribed
 *       control points of the updated state.  Note that since this function
 *       blocks on the mutex DSDevMutex, to avoid a hang this function should
 *       not be called within any other function that currently has this mutex
 *       locked.
 *
 * Parameters:
 *      num:var numbers
 *      service -- The service number (DS_SERVICE_Layer3Forwarding or DS_SERVICE_PICTURE)
 * 	varRec: var record list
 *   	-- var record structure
 *   		variable -- The variable number  
 *   		value -- The string representation of the new value
 *       2005.02.13 john qian
 ********************************************************************************/
int IGDDeviceSetServiceTableVars(int num, unsigned int service, struct VarRec *varrec) {
    int i;
    unsigned int variable;
    char *value;
    
    IXML_Document  *PropSet= NULL;
    
    if (service >= DS_SERVICE_SERVCOUNT)
	return(0);
    

    pthread_mutex_lock(&DSDevMutex);


    //Using second utility API
    for (i=0;i<num;i++) {
	variable=varrec[i].variable;
	value=varrec[i].value;
        strcpy(ds_service_table[service].VariableStrVal[variable], value);

	if (i==0)
            PropSet= UpnpCreatePropertySet(1,ds_service_table[service].VariableName[variable], ds_service_table[service].VariableStrVal[variable]);
	else
	    UpnpAddToPropertySet(&PropSet,ds_service_table[service].VariableName[variable], ds_service_table[service].VariableStrVal[variable]);
    }
	
	UpnpNotifyExt(gate.device_handle, ds_service_table[service].UDN, ds_service_table[service].ServiceId,PropSet);
        ixmlDocument_free(PropSet);
    
    pthread_mutex_unlock(&DSDevMutex);

    return(1);

}



/********************************************************************************
 * IGDDeviceCallbackEventHandler
 *
 * Description:
 *       The callback handler registered with the SDK while registering
 *       root device or sending a search request.  Detects the type of
 *       callback, and passes the request on to the appropriate procedure.
 *
 * Parameters:
 *   EventType -- The type of callback event
 *   Event -- Data structure containing event data
 *   Cookie -- Optional data specified during callback registration
 *
 ********************************************************************************/
int IGDDeviceCallbackEventHandler(Upnp_EventType EventType,
			 void *Event,
			 void *Cookie)
{
    /* Print a summary of the event received */
    SampleUtil_PrintEvent(EventType, Event);

    switch ( EventType) {
		case UPNP_EVENT_SUBSCRIPTION_REQUEST:
			IGDDeviceHandleSubscriptionRequest((struct Upnp_Subscription_Request *) Event);
			break;

    	case UPNP_CONTROL_GET_VAR_REQUEST:
			IGDDeviceHandleGetVarRequest((struct Upnp_State_Var_Request *) Event);
			break;

    	case UPNP_CONTROL_ACTION_REQUEST:
			IGDDeviceHandleActionRequest((struct Upnp_Action_Request *) Event);
			break;
		
		default:
			break;
    }

    return 0;
}
int IGDdelAllPortForwarding(void)
{

	DLL tmp;
	int i,len=LinkLength();

	//remove all Portmapping entry exist in both netfilter table and linked list
	for(i=len-1;i>=0;i--)
	{
			tmp=Search(i);
	       		if(tmp->data.PortMappingEnabled)
		       		delPortForward(tmp->data.Protocol,
       				getIPAddress(gate.ext_ifname)?: "0.0.0.0",
       				tmp->data.ExternalPort,
       				tmp->data.InternalClient,
       				tmp->data.InternalPort,
       				tmp->data.RemoteHost);

			Delete(tmp);
			print_link();
	}

   return (0);
}
