#ifndef _GATEWAY_H
#define _GATEWAY_H

#include "upnp.h"

#define DISALLOW	0	/* 0  not grant to change ; 1 grant */
#define DEBUG   0



/* SERVICE */
#define DS_SERVICE_SERVCOUNT 		    6
#define DS_SERVICE_OSInfo		    0  		/* Add to compatible for MS XP */
#define DS_SERVICE_WANCommonInterfaceConfig 1
#define DS_SERVICE_WANIPConnection	    2
#define DS_SERVICE_Layer3Forwarding                 3
#define DS_SERVICE_WANEthernetLinkConfig            4
#define DS_SERVICE_LANHostConfigManagement          5



/* OSInfo variable */
#define DS_OSI_VARCOUNT				4
#define DS_OSI_OSMajorVersion 			0
#define DS_OSI_OSMinorVersion			1
#define DS_OSI_OSBuildNumber			2
#define DS_OSI_OSMachineName			3

/*CommmonInterfaceConfig variable*/
#define DS_WANCIC_VARCOUNT 			16
#define DS_WANCIC_WANAccessType			0       /* DSL / POTS / Cable / Ethernet */
#define DS_WANCIC_Layer1UpstreamMaxBitRate	1
#define DS_WANCIC_Layer1DownstreamMaxBitRate	2
#define DS_WANCIC_PhysicalLinkStatus		3	/* Up / Down / Initializing / Unavailable */
#define DS_WANCIC_WANAccessProvider		4
#define DS_WANCIC_MaximumActiveConnections  	5
#define DS_WANCIC_NumberOfActiveConnections	6
#define DS_WANCIC_ActiveConnectionDeviceContainer 7
#define DS_WANCIC_ActiveConnectionServiceID	8
#define DS_WANCIC_TotalBytesSent		9
#define DS_WANCIC_TotalBytesReceived		10	
#define DS_WANCIC_TotalPacketsSent		11
#define DS_WANCIC_TotalPacketsReceived		12
#define DS_WANCIC_X_PersonalFirewallEnabled     13
#define DS_WANCIC_X_Uptime			14
#define DS_WANCIC_EnableForInternet		15	/* Enable / Disable */



/* WANIPConnection variable */

#define DS_WANIPConn_VARCOUNT		 	19

#define DS_WANIPConn_ConnectionType		0
#define DS_WANIPConn_PossibleConnectionTypes    1
#define DS_WANIPConn_ConnectionStatus		2
#define DS_WANIPConn_Uptime			3
#define DS_WANIPConn_RSIPAvailable		4
#define DS_WANIPConn_NATEnabled			5
#define DS_WANIPConn_X_Name			6
#define DS_WANIPConn_LastConnectionError	7
#define DS_WANIPConn_ExternalIPAddress		8
#define DS_WANIPConn_RemoteHost			9
#define DS_WANIPConn_ExternalPort		10
#define DS_WANIPConn_InternalPort		11
#define DS_WANIPConn_PortMappingProtocol	12	/* TCP or UDP */
#define DS_WANIPConn_InternalClient		13
#define DS_WANIPConn_PortMappingDescription	14	/* String */
#define DS_WANIPConn_PortMappingEnabled		15
#define DS_WANIPConn_PortMappingLeaseDuration	16
#define DS_WANIPConn_X_PortMappingIndex		17
#define DS_WANIPConn_PortMappingNumberOfEntries 18


/* Layer3Forwarding variable */
#define DS_L3F_VARCOUNT	                    1
#define DS_L3F_DefaultConnService           0

/* WANEthernetLinkConfig variable */
#define DS_WANELC_VARCOUNT	                        1
#define DS_WANELC_EthernetLinkStatus            0

/* LANHostConfigManagement variable */
#define DS_LANHostCM_VARCOUNT                           9
#define DS_LANHostCM_DHCPRelay                          0
#define DS_LANHostCM_DHCPSrvConfig                      1
#define DS_LANHostCM_DNSServers                         2
#define DS_LANHostCM_DomainName                         3
#define DS_LANHostCM_IPRouters                          4
#define DS_LANHostCM_MaxAddress                         5
#define DS_LANHostCM_MinAddress                         6
#define DS_LANHostCM_ReservedAddr                       7
#define DS_LANHostCM_SubnetMask                         8



#define DS_MAX_VAL_LEN 50

/* This should be the maximum VARCOUNT from above */
#define DS_MAXVARS DS_WANIPConn_VARCOUNT

extern char *IGDDeviceType[];
extern char *IGDServiceType[];


/* Structure for storing IGD Service identifiers and state table */
struct IGDService {
    char UDN[NAME_SIZE]; /* Universally Unique Device Name */
    char ServiceId[NAME_SIZE];
    char ServiceType[NAME_SIZE];
    char *VariableName[DS_MAXVARS]; 
    char *VariableStrVal[DS_MAXVARS];
    int  VariableCount;
};

/*Structre for storing TableVar index and value
 *   used in IGDDeviceSetServiceTableVars  
 *    2005.2.13 john qian
 * */
struct VarRec {
	unsigned int variable;
	char *value;
};


extern struct IGDService ds_service_table[];


/* Mutex for protecting the global state table data
   in a multi-threaded, asynchronous environment.
   All functions should lock this mutex before reading
   or writing the state table data. */
extern pthread_mutex_t DSDevMutex;


void IGDDeviceShutdownHdlr(int);

int IGDDeviceStateTableInit(char*,char*);
int IGDDeviceHandleSubscriptionRequest(struct Upnp_Subscription_Request *);
int IGDDeviceHandleGetVarRequest(struct Upnp_State_Var_Request *);
int IGDDeviceHandleActionRequest(struct Upnp_Action_Request *);
int IGDDeviceCallbackEventHandler(Upnp_EventType, void*, void*);


int IGDDeviceSetServiceTableVar(unsigned int, unsigned int, char*);
int IGDDeviceSetServiceTableVars(int,unsigned int, struct VarRec *);
int IGDdelAllPortForwarding(void);

#endif
