/*
 ***************************************************************************
 * Ralink Tech Inc.
 * 4F, No. 2 Technology 5th Rd.
 * Science-based Industrial Park
 * Hsin-chu, Taiwan, R.O.C.
 *
 * (c) Copyright, Ralink Technology, Inc.
 *
 * All rights reserved. Ralink's source code is an unpublished work and the
 * use of a copyright notice does not imply otherwise. This source code
 * contains confidential trade secret material of Ralink Tech. Any attemp
 * or participation in deciphering, decoding, reverse engineering or in any
 * way altering the source code is stricitly prohibited, unless the prior
 * written consent of Ralink Technology, Inc. is obtained.
 ***************************************************************************
 */

#ifndef __RT2880_GPIO_H__
#define __RT2880_GPIO_H__


/*
 * ioctl commands
 */
#define	RT2880GPIO_SET_DIR		0x01
#define RT2880GPIO_SET_DIR_IN		0x11
#define RT2880GPIO_SET_DIR_OUT		0x12
#define	RT2880GPIO_READ		0x02
#define	RT2880GPIO_WRITE		0x03
#define	RT2880GPIO_SET			0x21
#define	RT2880GPIO_CLEAR		0x31
#define	RT2880GPIO_READ_BIT		0x04
#define	RT2880GPIO_WRITE_BIT		0x05
#define	RT2880GPIO_READ_BYTE		0x06
#define	RT2880GPIO_WRITE_BYTE		0x07
#define	RT2880GPIO_READ_INT		0x02 //same as read
#define	RT2880GPIO_WRITE_INT		0x03 //same as write
#define	RT2880GPIO_SET_INT		0x21 //same as set
#define	RT2880GPIO_CLEAR_INT		0x31 //same as clear
#define RT2880GPIO_ENABLE_INTP		0x08
#define RT2880GPIO_DISABLE_INTP	0x09
#define RT2880GPIO_REG_IRQ		0x0A
#define RT2880GPIO_LED_SET		0x41

#define FLASH_MAX_RW_SIZE		0x100


/*
 * Address of RT2880 Registers
 */
#define RT2880_SYSCTL_ADDR		0xA0300000 // system control
#define RT2880_REG_GPIOMODE		(RT2880_SYSCTL_ADDR + 0x60)

#define RT2880_IRQ_ADDR			0xA0300200
#define RT2880_REG_INTENA		(RT2880_IRQ_ADDR + 0x34)
#define RT2880_REG_INTDIS		(RT2880_IRQ_ADDR + 0x38)

#define RT2880_PRGIO_ADDR		0xA0300600 // Programmable I/O
#define RT2880_REG_PIOINT		(RT2880_PRGIO_ADDR + 0)
#define RT2880_REG_PIOEDGE		(RT2880_PRGIO_ADDR + 0x04)
#define RT2880_REG_PIORENA		(RT2880_PRGIO_ADDR + 0x08)
#define RT2880_REG_PIOFENA		(RT2880_PRGIO_ADDR + 0x0C)
#define RT2880_REG_PIODATA		(RT2880_PRGIO_ADDR + 0x20)
#define RT2880_REG_PIODIR		(RT2880_PRGIO_ADDR + 0x24)
#define RT2880_REG_PIOSET		(RT2880_PRGIO_ADDR + 0x2C)
#define RT2880_REG_PIORESET		(RT2880_PRGIO_ADDR + 0x30)

/* for GPIO40 ~ GPIO71 */
#define RT2880_REG_PIO2INT		(RT2880_PRGIO_ADDR + 0x60)
#define RT2880_REG_PIO2EDGE		(RT2880_PRGIO_ADDR + 0x64)
#define RT2880_REG_PIO2RENA		(RT2880_PRGIO_ADDR + 0x68)
#define RT2880_REG_PIO2FENA		(RT2880_PRGIO_ADDR + 0x6C)
#define RT2880_REG_PIO2DATA		(RT2880_PRGIO_ADDR + 0x70)
#define RT2880_REG_PIO2DIR		(RT2880_PRGIO_ADDR + 0x74)

/*
 * Values for the GPIOMODE Register
 */
#define RT2880_GPIOMODE_I2C		0x01
#define RT2880_GPIOMODE_UARTF		0x02
#define RT2880_GPIOMODE_SPI		0x04
#define RT2880_GPIOMODE_UARTL		0x08
#define RT2880_GPIOMODE_JTAG		0x10
#define RT2880_GPIOMODE_MDIO		0x20
#define RT2880_GPIOMODE_SDRAM		0x40
#define RT2880_GPIOMODE_PCI		0x80
// if you would like to enable GPIO mode for other pins, please modify this value
// !! Warning: changing this value may make other features(MDIO, PCI, etc) lose efficacy
#define RT2880_GPIOMODE_DFT		(RT2880_GPIOMODE_UARTF)

/*
 * bit is the unit of length
 */
#define RT2880GPIO_NUMBER		24
#define RT2880GPIO_DATA_MASK		0x00FFFFFF
#define RT2880GPIO_DATA_LEN		24
#define RT2880GPIO_DIR_IN		0
#define RT2880GPIO_DIR_OUT		1
#define RT2880GPIO_DIR_ALLIN		0
#define RT2880GPIO_DIR_ALLOUT		0x00FFFFFF

/*
 * structure used at regsitration
 */
#define rt2880gpio_reg_info ralink_gpio_reg_info

#define RT2880GPIO_LED_LOW_ACT		1
#define RT2880GPIO_LED_INFINITY	4000
#define rt2880gpio_led_info ralink_gpio_led_info


#define RT2880_GPIO_0			0x00000001
#define RT2880_GPIO_1			0x00000002
#define RT2880_GPIO_2			0x00000004
#define RT2880_GPIO_3			0x00000008
#define RT2880_GPIO_4			0x00000010
#define RT2880_GPIO_5			0x00000020
#define RT2880_GPIO_6			0x00000040
#define RT2880_GPIO_7			0x00000080
#define RT2880_GPIO_8			0x00000100
#define RT2880_GPIO_9			0x00000200
#define RT2880_GPIO_10			0x00000400
#define RT2880_GPIO_11			0x00000800
#define RT2880_GPIO_12			0x00001000
#define RT2880_GPIO_13			0x00002000
#define RT2880_GPIO_14			0x00004000
#define RT2880_GPIO_15			0x00008000
#define RT2880_GPIO_16			0x00010000
#define RT2880_GPIO_17			0x00020000
#define RT2880_GPIO_18			0x00040000
#define RT2880_GPIO_19			0x00080000
#define RT2880_GPIO_20			0x00100000
#define RT2880_GPIO_21			0x00200000
#define RT2880_GPIO_22			0x00400000
#define RT2880_GPIO_23			0x00800000
#define RT2880_GPIO(x)			(1 << x)

#endif
